<?php namespace Terrostar\IcsFormExtension\Handler\Command;

use Anomaly\ConfigurationModule\Configuration\Contract\ConfigurationRepositoryInterface;
use Anomaly\FilesModule\File\Command\GetFile;
use Anomaly\FilesModule\File\Contract\FileInterface;
use Anomaly\FormsModule\Form\Contract\FormInterface;
use Anomaly\FormsModule\Form\FormAutoresponder;
use Anomaly\FormsModule\Form\FormMailer;
use Anomaly\Streams\Platform\Support\Template;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Mail\Message;
use Illuminate\Routing\Redirector;
use Terrostar\IcsFormExtension\Handler\IcsFormBuilder;

class GetIcsFormBuilder
{

    use DispatchesJobs;

    /**
     * The form instance.
     *
     * @var FormInterface
     */
    protected $form;

    /**
     * Create a new GetIcsFormBuilder instance.
     *
     * @param FormInterface $form
     */
    public function __construct(FormInterface $form)
    {
        $this->form = $form;
    }

    /**
     * Handle the command.
     *
     * @param IcsFormBuilder $builder
     * @return IcsFormBuilder
     */
    public function handle(IcsFormBuilder $builder, Redirector $redirect)
    {
        $stream = $this->form->getFormEntriesStream();

        $builder->on(
            'saved',
            function (FormMailer $mailer, FormAutoresponder $autoresponder) use ($builder) {
                $mailer->send($this->form, $builder);
                $autoresponder->send($this->form, $builder);
            }
        );

        $builder->on(
            'attaching_files',
            function (
                ConfigurationRepositoryInterface $configuration,
                Template $template,
                Message $message
            ) use ($builder) {

                /* @var FileInterface $file */
                $file = $this->dispatch(
                    new GetFile(
                        array_get(
                            $configuration->findByKeyAndScopeOrNew(
                                $this->form->getFormHandler()->getNamespace('ics_file'),
                                $this->form->getId()
                            )->getAttributes(),
                            'value'
                        )
                    )
                );

                $message->attachData($file->resource()->read(), $file->getName());
            }
        );

        return $builder
            ->setActions(['submit'])
            ->setModel($stream->getEntryModel())
            ->setOption('panel_class', 'section')
            ->setOption('enable_defaults', false)
            ->setOption('success_message', $this->form->getSuccessMessage() ?: false)
            ->setOption('redirect', $this->form->getSuccessRedirect() ?: $redirect->back());
    }
}
