<?php namespace Terrostar\IcsFormExtension;

use Anomaly\ConfigurationModule\Configuration\Contract\ConfigurationRepositoryInterface;
use Anomaly\FormsModule\Form\Contract\FormInterface;
use Anomaly\FormsModule\Form\Handler\Contract\FormHandlerExtensionInterface;
use Anomaly\Streams\Platform\Addon\Extension\Extension;
use Anomaly\Streams\Platform\Ui\Form\FormBuilder;
use Terrostar\IcsFormExtension\Handler\Command\GetIcsFormBuilder;

class IcsFormExtension extends Extension implements FormHandlerExtensionInterface
{

    /**
     * @var null|string
     */
    protected $provides = 'anomaly.module.forms::form_handler.ics';

    /**
     * Return the form's builder instance.
     *
     * @param FormInterface $form
     * @return FormBuilder
     */
    public function builder(FormInterface $form)
    {
        return $this->dispatch(new GetIcsFormBuilder($form));
    }

    /**
     * Integrate the form handler's services
     * with the primary form's builder instance.
     *
     * @param FormBuilder $builder
     */
    public function integrate(FormBuilder $builder)
    {
        /* @var ConfigurationRepositoryInterface $configuration */
        $configuration = app(ConfigurationRepositoryInterface::class);

        $value = array_get(
            $configuration->findByKeyAndScopeOrNew(
                $this->getNamespace('ics_file'),
                request('id')
            )->getAttributes(),
            'value'
        );

        $builder->addField(
            'ics_file',
            [
                'required' => true,
                'save'     => false,
                'value'    => $value,
                'label'    => 'ICS File',
                'type'     => 'anomaly.field_type.file',
            ]
        );

        $builder->addSection(
            'ics',
            [
                'fields' => [
                    'ics_file',
                ],
            ]
        );

        $builder->on(
            'saved',
            function () use ($configuration, $builder) {
                $configuration->set(
                    $this->getNamespace('ics_file'),
                    $builder->getFormEntryId(),
                    $builder->getPostValue('ics_file')
                );
            }
        );

        return $builder;
    }
}
